/**
 * @module MailcowClient
 */
import { AxiosRequestConfig } from 'axios';
import { DomainEndpoints } from './Endpoints/domain-endpoints';
import { AntiSpamEndpoints } from './Endpoints/antispam-endpoints';
import { MailboxEndpoints } from './Endpoints/mailbox-endpoint';
import RequestFactory from './request-factory';
import { AliasEndpoints } from './Endpoints/alias-endpoints';
import { SyncjobEndpoints } from './Endpoints/syncjob-endpoints';
import { ForwardingEndpoints } from './Endpoints/forwarding-endpoints';
import { LogEndpoints } from './Endpoints/log-endpoints';
/**
 * Class containing all the logic to interface with the Mailcow API in TypeScript.
 * @external
 */
declare class MailcowClient {
    /**
     * The base URL of the Mailcow API.
     */
    readonly BASE_URL: string;
    /**
     * The API key of the Mailcow API.
     */
    readonly API_KEY: string;
    /**
     * The headers used for every request.
     * @internal
     */
    HEADERS: AxiosRequestConfig;
    /**
     * Creates a MailcowClient using the given URL and API key.
     * @param BASE_URL - The base URL of the Mailcow API.
     * @param API_KEY  - The API key of the Mailcow API.
     * @param EXTRA_AXIOS_CONFIG - Allows for setting extra Axios request config such as keep alive.
     */
    constructor(BASE_URL: string, API_KEY: string, EXTRA_AXIOS_CONFIG?: AxiosRequestConfig);
    /**
     * Factory method pattern for creating HTTP requests.
     * @internal
     */
    requestFactory: RequestFactory;
    /**
     * All endpoints related to Aliases.
     * See {@link AliasEndpoints}
     * @external
     */
    aliases: AliasEndpoints;
    /**
     * All endpoints related to Domains.
     * See {@link DomainEndpoints}
     * @external
     */
    domains: DomainEndpoints;
    /**
     * All endpoints related to spam policies.
     * See {@link AntiSpamEndpoints}
     * @external
     */
    spamPolicy: AntiSpamEndpoints;
    /**
     * All endpoints related to mailboxes.
     * See {@link MailboxEndpoints}
     * @external
     */
    mailbox: MailboxEndpoints;
    /**
     * All endpoints related to sync jobs.
     * See {@link SyncjobEndpoints}
     * @external
     */
    syncjobs: SyncjobEndpoints;
    /**
     * All endpoints related to forwarding hosts.
     * See {@link ForwardingEndpoints}
     * @external
     */
    forwardingHosts: ForwardingEndpoints;
    /**
     * All endpoints related to logs.
     * See {@link LogEndpoints}
     * @external
     */
    logs: LogEndpoints;
}
export default MailcowClient;
