"use strict";
/**
 * @module MailcowClient
 */
Object.defineProperty(exports, "__esModule", { value: true });
const domain_endpoints_1 = require("./Endpoints/domain-endpoints");
const antispam_endpoints_1 = require("./Endpoints/antispam-endpoints");
const mailbox_endpoint_1 = require("./Endpoints/mailbox-endpoint");
const request_factory_1 = require("./request-factory");
const alias_endpoints_1 = require("./Endpoints/alias-endpoints");
const syncjob_endpoints_1 = require("./Endpoints/syncjob-endpoints");
const forwarding_endpoints_1 = require("./Endpoints/forwarding-endpoints");
const log_endpoints_1 = require("./Endpoints/log-endpoints");
/**
 * Class containing all the logic to interface with the Mailcow API in TypeScript.
 * @external
 */
class MailcowClient {
    /**
     * Creates a MailcowClient using the given URL and API key.
     * @param BASE_URL - The base URL of the Mailcow API.
     * @param API_KEY  - The API key of the Mailcow API.
     * @param EXTRA_AXIOS_CONFIG - Allows for setting extra Axios request config such as keep alive.
     */
    constructor(BASE_URL, API_KEY, EXTRA_AXIOS_CONFIG) {
        /**
         * Factory method pattern for creating HTTP requests.
         * @internal
         */
        this.requestFactory = new request_factory_1.default(this);
        /**
         * All endpoints related to Aliases.
         * See {@link AliasEndpoints}
         * @external
         */
        this.aliases = (0, alias_endpoints_1.aliasEndpoints)(this);
        /**
         * All endpoints related to Domains.
         * See {@link DomainEndpoints}
         * @external
         */
        this.domains = (0, domain_endpoints_1.domainEndpoints)(this);
        /**
         * All endpoints related to spam policies.
         * See {@link AntiSpamEndpoints}
         * @external
         */
        this.spamPolicy = (0, antispam_endpoints_1.antiSpamEndpoints)(this);
        /**
         * All endpoints related to mailboxes.
         * See {@link MailboxEndpoints}
         * @external
         */
        this.mailbox = (0, mailbox_endpoint_1.mailboxEndpoints)(this);
        /**
         * All endpoints related to sync jobs.
         * See {@link SyncjobEndpoints}
         * @external
         */
        this.syncjobs = (0, syncjob_endpoints_1.syncjobEndpoints)(this);
        /**
         * All endpoints related to forwarding hosts.
         * See {@link ForwardingEndpoints}
         * @external
         */
        this.forwardingHosts = (0, forwarding_endpoints_1.forwardingEndpoints)(this);
        /**
         * All endpoints related to logs.
         * See {@link LogEndpoints}
         * @external
         */
        this.logs = (0, log_endpoints_1.logEndpoints)(this);
        this.BASE_URL = BASE_URL.charAt(BASE_URL.length - 1) === '/' ? BASE_URL : BASE_URL.concat('/');
        this.API_KEY = API_KEY;
        // Set the correct Axios headers.
        this.HEADERS = Object.assign({ headers: {
                'Content-Type': 'application/json',
                'X-API-Key': this.API_KEY
            } }, EXTRA_AXIOS_CONFIG);
    }
}
exports.default = MailcowClient;
//# sourceMappingURL=index.js.map